<?php

define('PAGE_NOT_FOUND', 'Stránka nenalezena');

class BaseView extends View
{
  public bool $FormatHTML = false;
  public bool $ShowRuntimeInfo = false;
  public bool $RawPage = false;
  public bool $BasicHTML = false;
  public string $ParentClass = '';
  public string $Title;
  public string $Description;
  public string $Encoding;
  public string $Style;

  function __construct(System $System)
  {
    parent::__construct($System);

    $this->FormatHTML = false;
    $this->ShowRuntimeInfo = false;
    $this->Encoding = 'utf-8';
    $this->Style = 'new';
  }

  function SystemMessage(string $Title, string $Text): string
  {
    return '<table align="center"><tr><td><div class="SystemMessage"><h3>'.$Title.'</h3><div>'.$Text.'</div></div></td></tr></table>';
    //ShowFooter();
    //die();
  }

  function ShowNavigation(Page $Page): string
  {
    if (array_key_exists('REQUEST_URI', $_SERVER))
      $ScriptName = $_SERVER['REQUEST_URI'];
      else $ScriptName = '';
    while (strpos($ScriptName, '//') !== false)
      $ScriptName = str_replace('//', '/', $ScriptName);
    if (strpos($ScriptName, '?') !== false)
      $ScriptName = substr($ScriptName, 0, strrpos($ScriptName, '?'));
    $ScriptName = substr($ScriptName, strlen($this->System->Link('')));
    if (substr($ScriptName, -1, 1) == '/') $ScriptName = substr($ScriptName, 0, -1);

    $Output = '';
    while ($Page)
    {
      $Output = ' &gt; <a href="'.$this->System->Link($ScriptName).'/">'.$Page->Title.'</a>'.$Output;

      if (($Page->ParentClass != null) and class_exists($Page->ParentClass))
      {
        $PageClass = $Page->ParentClass;
        $Page = new $PageClass($this->System);
        $ScriptName = substr($ScriptName, 0, strrpos($ScriptName, '/'));
      } else $Page = null;
    }
    $Output = substr($Output, 6);
    return $Output;
  }

  function ShowHeader(Page $Page): string
  {
    $Path = $Page->Title;

    $Navigation = $this->ShowNavigation($Page);

    $BodyParam = '';
    if (isset($Page->Load)) $BodyParam .= ' onload="'.$Page->Load.'"';
    if (isset($Page->Unload)) $BodyParam .= ' onunload="'.$Page->Unload.'"';
    $Output = '<?xml version="1.0" encoding="'.$this->Encoding.'"?>'."\n".
    '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">'.
    '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="cs" lang="cs">'.
    '<head><link rel="stylesheet" href="'.$this->System->Link('/style/').$this->Style.'/style.css" type="text/css" media="all" />'.
    '<meta http-equiv="content-type" content="application/xhtml+xml; charset='.$this->Encoding.'" />'.
    '<script type="text/javascript" src="'.$this->System->Link('/style/').$this->Style.'/global.js"></script>'.
    '<title>'.$Path.' - '.$this->System->Config['Web']['Title'].'</title>';

    // Show page headers
    $Bar = '';
    foreach ($this->System->PageHeaders as $Item)
      $Output .= call_user_func($Item);

    $Output .= '</head><body'.$BodyParam.'>';
    if ($this->BasicHTML == false)
    {
      //$Output .= '<div class="MainTitle">'.$Title.'</div>';
      $Output .= '<div class="MainTitle"><span class="MenuItem"><strong>Navigace :: </strong> '.$Navigation.'</span><div class="MenuItem2">';
      $Bar = '';
      foreach ($this->System->Bars['Top'] as $BarItem)
        $Bar .= call_user_func($BarItem);
      if (trim($Bar) != '') $Output .= $Bar;
        else $Output .= '&nbsp;';
      $Output .= '</div></div>';
    }
    return $Output;
  }

  function ShowFooter(): string
  {
    global $Revision, $ReleaseTime;

    $Time = round(GetMicrotime() - $this->System->ScriptTimeStart, 2);
    $Output = '';
    if ($this->BasicHTML == false)
    {
      $Output .= '<div id="Footer">'.
      '<i>| Správa webu: '.$this->System->Config['Web']['Admin'].' | e-mail: '.$this->System->Config['Web']['AdminEmail'].' | '.
      ' Verze: '.$Revision.' ('.Core::Cast($this->System)->HumanDate($ReleaseTime).') |';
      if ($this->ShowRuntimeInfo == true) $Output .= ' Doba generování: '.$Time.' s / '.ini_get('max_execution_time').
        ' s | Použitá paměť: '.HumanSize(memory_get_peak_usage(FALSE)).' / '.ini_get('memory_limit').'B |';
      $Output .= '</i></div>';
    }
    $Output .= '</body></html>';
    return $Output;
  }

  function GetOutput(Page $Page): string
  {
    $Page->OnSystemMessage = array($this->System->BaseView, 'SystemMessage');

    $Output = $Page->Show();
    if ($Page->RawPage == false)
    {
      $Output = $this->ShowHeader($Page).$Output.$this->ShowFooter();
      if ($this->FormatHTML == true) $Output = $this->FormatOutput($Output);
    }
    return $Output;
  }

  function NewPage(string $ClassName): Page
  {
    $Page = new $ClassName();
    $Page->System = $this->System;
    $Page->Database = $this->Database;
    $Page->FormatHTML = $this->FormatHTML;
    return $Page;
  }

  // XML formating function
  function FormatOutput(string $s): string
  {
    $out = '';
    $nn = 0;
    $n = 0;
    while ($s != '')
    {
      $start = strpos($s, '<');
      $end = strpos($s, '>');
      if ($start != 0)
      {
        $end = $start - 1;
        $start = 0;
      }
      $line = trim(substr($s, $start, $end + 1));
      if (strlen($line) > 0)
      if ($line[0] == '<')
      {
        if ($s[$start + 1] == '/')
        {
          $n = $n - 2;
          $nn = $n;
        } else
        {
          if (strpos($line, ' ')) $cmd = substr($line, 1, strpos($line, ' ') - 1);
          else $cmd = substr($line, 1, strlen($line) - 2);          
          if (strpos($s, '</'.$cmd.'>')) $n = $n + 2;
        }
      }
      if ($line != '') $out .= (str_repeat(' ', $nn).$line."\n");
      $s = substr($s, $end + 1, strlen($s));
      $nn = $n;
    }
    return $out;
  }
}
