<?php

include_once(dirname(__FILE__).'/Base.php');

class TypeDateTime extends TypeBase
{
  function __construct(FormManager $FormManager)
  {
    parent::__construct($FormManager);
    $this->DatabaseCompareOperators = array('Rovno' => '=', 'Nerovno' => '!=', 'Menší' => '<', 'Větší' => '>');
  }

  function OnView(array $Item): ?string
  {
    if ($Item['Value'] == 0) return '';
    if ((strtolower($Item['Value']) == 'now') or (strtolower($Item['Value']) == '')) $Item['Value'] = time();
    $Parts = getdate($Item['Value']);
    $Output = $Parts['mday'].'.'.$Parts['mon'].'.'.$Parts['year'].' '.
      sprintf('%02d', $Parts['hours']).':'.sprintf('%02d', $Parts['minutes']).':'.sprintf('%02d', $Parts['seconds']);
    return $Output;
  }

  function OnEdit(array $Item): string
  {
    global $MonthNames;

    if (($Item['Value'] == null) or (($Item['Value'] !== null) and ((strtolower($Item['Value']) == 'now') or (strtolower($Item['Value']) == ''))))
    {
      $Item['Value'] = time();
      $IsNull = true;
    } else $IsNull = false;
    $Parts = getdate($Item['Value']);

    $Output = '';
    $Style = '';
    if (array_key_exists('Null', $Item) and $Item['Null'])
    {
      if (!$IsNull)
      {
        $Checked = ' checked="1"';
        $Style = 'style="display:inline;"';
      } else
      {
        $Checked = '';
        $Style = 'style="display:none;"';
      }
      $Output .= '<input type="checkbox" name="'.$Item['Name'].'-null"'.$Checked.' onclick="toggle(\''.
          $Item['Name'].'-hour\');toggle(\''.$Item['Name'].'-minute\');toggle(\''.$Item['Name'].'-second\');toggle(\''.
          $Item['Name'].'-day\');toggle(\''.$Item['Name'].'-month\');toggle(\''.$Item['Name'].'-year\');"/>';
    }

    // Hour
    $Output .= '<select name="'.$Item['Name'].'-hour" id="'.$Item['Name'].'-hour" '.$Style.'>';
    for ($I = 1; $I <= 24; $I++)
    {
      if ($Parts['hours'] == $I) $Selected = ' selected="1"'; else $Selected = '';
      $Output .= '<option value="'.$I.'"'.$Selected.'>'.$I.'</option>';
    }
    $Output .= '</select>';
    // Minute
    $Output .= '<select name="'.$Item['Name'].'-minute" id="'.$Item['Name'].'-minute" '.$Style.'>';
    for ($I = 1; $I <= 60; $I++)
    {
      if ($Parts['minutes'] == $I) $Selected = ' selected="1"'; else $Selected = '';
      $Output .= '<option value="'.$I.'"'.$Selected.'>'.$I.'</option>';
    }
    $Output .= '</select>';
    // Second
    $Output .= '<select name="'.$Item['Name'].'-second" id="'.$Item['Name'].'-second" '.$Style.'>';
    for ($I = 1; $I <= 60; $I++)
    {
      if ($Parts['seconds'] == $I) $Selected = ' selected="1"'; else $Selected = '';
      $Output .= '<option value="'.$I.'"'.$Selected.'>'.$I.'</option>';
    }
    $Output .= '</select>';
    // Day
    $Output .= ' <select name="'.$Item['Name'].'-day" id="'.$Item['Name'].'-day" '.$Style.'>';
    for ($I = 1; $I <= 31; $I++)
    {
      if ($Parts['mday'] == $I) $Selected = ' selected="1"'; else $Selected = '';
      $Output .= '<option value="'.$I.'"'.$Selected.'>'.$I.'</option>';
    }
    $Output .= '</select>';
    // Month
    $Output .= '<select name="'.$Item['Name'].'-month"  id="'.$Item['Name'].'-month" '.$Style.'>';
    for ($I = 1; $I <= 12; $I++)
    {
      if ($Parts['mon'] == $I) $Selected = ' selected="1"'; else $Selected = '';
      $Output .= '<option value="'.$I.'"'.$Selected.'>'.$MonthNames[$I].'</option>';
    }
    $Output .= '</select>';
    // Year
    $Output .= '<select name="'.$Item['Name'].'-year" id="'.$Item['Name'].'-year" '.$Style.'>';
    for ($I = 1900; $I < 2100; $I++)
    {
      if ($Parts['year'] == $I) $Selected = ' selected="1"'; else $Selected = '';
      $Output .= '<option value="'.$I.'"'.$Selected.'>'.$I.'</option>';
    }
    $Output .= '</select>';
    return $Output;
  }

  function OnLoad(array $Item): ?string
  {
    if (!array_key_exists($Item['Name'].'-null', $_POST) and array_key_exists('Null', $Item) and ($Item['Null'] == true)) return null;
      else return mktime($_POST[$Item['Name'].'-hour'], $_POST[$Item['Name'].'-minute'], $_POST[$Item['Name'].'-second'],
        $_POST[$Item['Name'].'-month'], $_POST[$Item['Name'].'-day'], $_POST[$Item['Name'].'-year']);
  }

  function OnCanLoad(array $Item): bool
  {
    return array_key_exists($Item['Name'].'-null', $_POST) or (array_key_exists($Item['Name'].'-day', $_POST) and 
      array_key_exists($Item['Name'].'-month', $_POST) and array_key_exists($Item['Name'].'-year', $_POST) and
      array_key_exists($Item['Name'].'-hour', $_POST) and array_key_exists($Item['Name'].'-minute', $_POST) and
      array_key_exists($Item['Name'].'-second', $_POST)
    );
  }

  function OnLoadDb(array $Item): ?string
  {
    return MysqlDateTimeToTime($Item['Value']);
  }

  function OnSaveDb(array $Item): ?string
  {
    if ($Item['Value'] == null) return null;
      else return date('Y-m-d H:i:s', $Item['Value']);
  }

  function DatabaseEscape(string $Value): string
  {
    return '"'.addslashes($Value).'"';
  }
}
